﻿namespace AsyncSample3.ViewModels
{
    using System;
    using System.Threading;
    using System.Threading.Tasks;

    public class MainViewModel : NotificationObject
    {
        #region 公開プロパティ
        private string result = "まだ実行してないよ。";
        /// <summary>
        /// 実行結果を取得または設定します。
        /// </summary>
        public string Result
        {
            get { return result; }
            set { SetProperty(ref result, value); }
        }

        private bool isBusy;
        /// <summary>
        /// ビジー状態かどうかを取得または設定します。
        /// </summary>
        public bool IsBusy
        {
            get { return isBusy; }
            set
            {
                if (SetProperty(ref isBusy, value))
                    ButtonCommand.RaiseCanExecuteChanged();
            }
        }

        private DelegateCommand buttonCommand;
        /// <summary>
        /// ボタンコマンドを取得します。
        /// </summary>
        public DelegateCommand ButtonCommand
        {
            get
            {
                return buttonCommand ?? (buttonCommand = new DelegateCommand(
                    _ =>
                    {
                        System.Console.WriteLine("Thread[{0}] 非同期処理をコールします。", Thread.CurrentThread.ManagedThreadId);
                        HeavyWorkAsync();
                        System.Console.WriteLine("Thread[{0}] 非同期処理をコールしました。", Thread.CurrentThread.ManagedThreadId);
                    },
                    _ =>
                    {
                        return !IsBusy;
                    }));
            }
        }
        #endregion 公開プロパティ

        #region private メソッド
        /// <summary>
        /// 重たい処理を非同期で実行します。
        /// </summary>
        private async void HeavyWorkAsync()
        {
            IsBusy = true;
            Result = "只今実行中...";
            System.Console.WriteLine("Thread[{0}] 非同期処理を実行します。", Thread.CurrentThread.ManagedThreadId);
            await HeavyWorkTask();
            // ここで一旦 return される

            // 非同期処理が終了したらここから再開する
            System.Console.WriteLine("Thread[{0}] 非同期処理を実行しました。", Thread.CurrentThread.ManagedThreadId);
            Result = "終了しました。";
            IsBusy = false;
        }

        /// <summary>
        /// 重たい処理を実行するタスクを返します。
        /// </summary>
        /// <returns>重たい処理を実行するタスク</returns>
        private Task HeavyWorkTask()
        {
            return Task.Run(() =>
            {
                // 進捗報告用クラスのインスタンス生成
                var p = new Progress<int>(i =>
                {
                    // 進捗報告は UI スレッドで実行されるため、
                    // データバインドされたプロパティも操作できる
                    Result = "只今実行中..." + " [" + i.ToString() + "%]";
                });

                // 重たい処理を実行
                System.Console.WriteLine("Thread[{0}] 重たい処理を実行します。", Thread.CurrentThread.ManagedThreadId);
                HeavyWork(p);
                System.Console.WriteLine("Thread[{0}] 重たい処理を終了します。", Thread.CurrentThread.ManagedThreadId);
            });
        }

        /// <summary>
        /// 重たい処理を実行します。
        /// </summary>
        /// <param name="progress">進捗を示すための <code>System.IProgress&lt;int&gt;</code> を実装したクラスを指定します。</param>
        private void HeavyWork(IProgress<int> progress)
        {
            int count = 0;
            while (count++ < 100)
            {
                // 重たい処理
                Thread.Sleep(30);

                // 進捗率の更新
                progress.Report(count);
            }
        }
        #endregion private メソッド
    }
}
